/* $Id: beacon.h 321 2009-10-19 14:25:40Z oh2gve $
 * 
 * Copyright 2009 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef BEACON_HH
#define BEACON_HH


// Own stuff.
#include "aprspacket.h"
#include "connection.h"
#include "dupefilter.h"
#include "packetlog.h"

// std stuff.
#include <vector>
#include <string>
#include <ctime>
#include <map>



/// Duplicate controller.
class Beacon
{
 public:
      
   /// Constructor.
   Beacon(std::vector<Connection*> const& conns, Connection* uplink,
          Cfg::Configuration const& cfg,
          DupeFilter& dupefilter, Packetlog& packetlog);
   
   /// Destructor.
   ~Beacon();
   
   /// Tells how initialization went.
   bool operator!() const;
   
   /// Gives error message.
   std::string error() const;
   
   /// Checks if beaconing is necessary and does it.
   void kick();
   
 private:
   
   /// Beacon packet as stored in our beacon list.
   struct BeaconPacket
   {
      /// The actual packet to beacon.
      AprsPacket* packet;
      
      /// Connection where this packet is to be beaconed.
      Connection* conn;
      
      /// Number of conn in config file.
      unsigned int connNo;
      
      /// Beaconing interval in seconds.
      time_t interval;
   };
   
   /// List of beacon packets. Next beacon time is the key.
   std::multimap<time_t, BeaconPacket> beacons_;
   
   /// Duplicate contoller.
   DupeFilter& dupefilter_;
   
   /// Packet log.
   Packetlog& packetlog_;
   
   /// Template for outputting conns -part of log line.
   std::string const outcons_;
   
   /// If initialization failed, an error message will be available here.
   std::string error_;
};

#endif // BEACON_HH
