/* $Id: dupefilter.cpp 319 2009-10-19 14:06:42Z oh2gve $
 * 
 * Copyright 2009 Tapio Aaltonen
 *
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/


// Own stuff.
#include "dupefilter.h"
#include "aprspacket.h"

// std stuff.
#include <map>
using std::multimap;
using std::pair;
using std::make_pair;
#include <string>
using std::string;
#include <ctime>



DupeFilter::DupeFilter(time_t const& dupetime):
dupetime_(dupetime)
{
}



DupeFilter::~DupeFilter()
{
   for ( multimap<string const, AprsPacket const*>::iterator i = storage_.begin(); i != storage_.end(); ++i )
   {
      delete i->second;
   }
}



bool DupeFilter::isDupe(AprsPacket const* packet)
{
   // Check if this target has packets in storage.
   pair< multimap<string const, AprsPacket const*>::iterator,
         multimap<string const, AprsPacket const*>::iterator > packetRange = storage_.equal_range(packet->getLabel());

   if ( packetRange.second != packetRange.first )
   {
      // Check packets in the found range for exact match.
      for ( multimap<string const, AprsPacket const*>::iterator i = packetRange.first; i != packetRange.second; ++i )
      {
         if ( *i->second == *packet )
         {
            return true;
         }
      }
   }
   
   // Apparently not a dupe, save it in storage.
   storage_.insert( make_pair(packet->getLabel(), new AprsPacket(*packet)) );
   
   return false;
}

void DupeFilter::flush()
{   
   for ( multimap<string const, AprsPacket const*>::iterator i = storage_.begin(); i != storage_.end(); ++i )
   {
      if ( time(NULL) - i->second->getTimestamp() > dupetime_ )
      {
         delete i->second;
         storage_.erase(i);
         // We do this one at a time.
         return;
      }
   }
}

