/* $Id: dupefilter.h 319 2009-10-19 14:06:42Z oh2gve $
 * 
 * Copyright 2009 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef DUPEFILTER_HH
#define DUPEFILTER_HH


// Own stuff.
#include "aprspacket.h"

// std stuff.
#include <map>
#include <string>
#include <ctime>


/// Duplicate controller.
class DupeFilter
{
 public:
      
   /// Constructor.
   DupeFilter(time_t const& dupetime);
   
   /// Destructor.
   ~DupeFilter();
   
   /// Checks if given packet is a duplicate.
   bool isDupe(AprsPacket const* packet);
   
   /// Flushes packets older that dupetime from storage.
   /**
    * Call this regularly, or storage will contain too old packets.
    */
   void flush();
   
 private:
   
   /// List of duplicate packets.
   /**
    * Label of target in packet is stored as key to improve efficiency. This
    * might come handy if we're fed with full APRS-IS feed...
    */
   std::multimap<std::string const, AprsPacket const*> storage_;
   
   /// Amount of time to store packets in storage_.
   time_t const dupetime_;
};

#endif // DUPEFILTER_HH
