/* $Id: logfile.h 319 2009-10-19 14:06:42Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef LOGFILE_HH
#define LOGFILE_HH

// std stuff
#include <fstream>
#include <string>



/// Models a packet log file.
/**
 * Can be used always, however packets will be actually written into file
 * only if it was successfully opened.
 */
class Logfile
{
 public:
      
   /// Constructor.
   /**
    * \param filename Name of log file to write. If not given, no writing will
    * actually happen.
    */
   Logfile(char const* filename);
   
   /// Destructor.
   ~Logfile();
   
   /// Write given bytes to file.
   void write(std::string const& line);
   
   /// Status query.
   /**
    * \return true If a file was given, but it couldn't be opened.
    * \return false All other cases.
    */
   bool operator!() const;
     
 private:
   
   /// Name of this file.
   bool filenamegiven_;
   
   /// The log file itself.
   std::ofstream file_;
   
};

#endif // LOGFILE_HH
