/* $Id: logserver.h 322 2009-10-19 16:43:34Z oh2gve $
 *
 * Copyright 2009 Tapio Aaltonen
 *
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/


#ifndef LOGSERVER_HH
#define LOGSERVER_HH


// own stuff.
#include "cfg.h"
#include "aprspacket.h"

// std stuff.
#include <string>
#include <map>

// wx stuff.
#include <wx/thread.h>
#include <wx/socket.h>
#include <wx/string.h>


/// A log server.
class LogServer : public wxThread
{
 public:
   
   /// Constructor.
   /**
    * Params of this IGate are used in greeting message sent to every incoming
    * connection.
    * \param cfg Which iface to bind and which port to listen.
    * \param ownCall Callsign of this IGate.
    * \param ownLat Latitude from config file.
    * \param ownLon Longitude from config file.
    */
   LogServer(wxIPV4address const& cfg, wxString const& ownCall,
             AprsPacket::Location const& ownLoc);

   /// Destructor.
   virtual ~LogServer();

   /// This is the main function of the thread.
   virtual void* Entry();

   /// Blocks until the server has initialized or failed to initalize.
   /**
    * Should always be called right after Run().
    */
   void waitForStartup();

   /// Get status of the connection.
   /**
    * \return true The server has a fatal error.
    * \return false Everything is ok.
    */
   bool operator!();

   /// Write given line to server.
   /**
    * If server configuration was empty, this does nothing.
    */
   void write(std::string const& line);

 private:

   /// Prints error description in log.
   void logError(wxSocketError const& lastError) const;
     
   /// Turns true when startup operations are complete.
   bool startupDone_;
   
   /// True if startup was successfull.
   bool initialized_;
  
   /// Mutex for protecting startup of this thread.
   wxMutex mutex_;

   /// Server config.
   wxIPV4address addr_;
   
   /// Call of the IGate.
   wxString ownCall_;
   
   /// Location of the IGate.
   AprsPacket::Location ownLoc_;
   
   // Master socket.
   /**
    * Needs to be handled dynamically because wants wxIPV4address (which only
    * has default constructor) as a constructor parameter.
    */
   wxSocketServer* masterSocket_;
   
   /// List of connected clients.
   /**
    * Map key is string containing host:port.
    */
   typedef std::map<wxString, wxSocketBase*> ClientList;
   
   /// Client listing.
   ClientList clients_;
   
};

#endif // LOGSERVER_HH
