/* $Id: packetlog.cpp 322 2009-10-19 16:43:34Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009 Tapio Aaltonen
 *
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

// own stuff.
#include "packetlog.h"
#include "cfg.h"
using Cfg::LogCfg;
#include "logserver.h"
#include "packet.h"
#include "aprspacket.h"

// std stuff.
#include <string>
using std::string;
#include <list>
using std::list;
#include <sstream>
using std::ostringstream;
#include <iomanip>

// wx stuff.
#include <wx/string.h>
#include <wx/datetime.h>
#include <wx/log.h>


Packetlog::Packetlog(LogCfg const& cfg, wxString const& ownCall,
                     AprsPacket::Location const& ownLoc):
initialized_(true), cfg_(cfg), fileNeeded_(!cfg.packetFileName.empty()), file_(NULL), server_(NULL)
{
   // Start the server if needed.
   if ( cfg_.serverNeeded )
   {
      wxLogMessage(wxT(" packetlog: starting log server."));
      
      // Create the server thread.
      server_ = new LogServer(cfg.packetSrv, ownCall, ownLoc);
      if ( server_->Create() != wxTHREAD_NO_ERROR )
      {
         wxLogMessage(wxT(" packetlog: failed to create thread for packet log server."));
         initialized_ = false;
      }
      else
      {
         // Start the server thread.
         server_->Run();
      }
   
      // Make sure server thread has started up.
      if ( initialized_ )
      {
         server_->waitForStartup();
      }
      
      // Check for fatal errors.
      if ( initialized_ && !(*server_) )
      {
         wxLogMessage(wxT(" packetlog: log server could not be started."));
         initialized_ = false;
      }
   
      // Put note to general log.
      if ( initialized_ )
      {
         wxLogMessage(wxT(" packetlog: log server started."));
      }
   }
   
   // Open file if needed.
   if ( fileNeeded_ )
   {
      file_ = new Logfile(cfg_.packetFileName.ToAscii());
   
      // Check for fatal errors in file.
      if ( !(*file_) )
      {
         wxLogMessage(wxT(" packetlog: file ") + cfg_.packetFileName + wxT(" could not be opened."));
         initialized_ = false;
      }
   
      // Put note to general log if file was opened.
      if ( initialized_ )
      {
         wxLogMessage(wxT(" packetlog: log file opened."));
      }
   }
   
   // Tell that we're go.
   if ( cfg_.serverNeeded || fileNeeded_ )
   {
      wxLogMessage(wxT("Packet logging started."));
   }
}

Packetlog::~Packetlog()
{
   // Close server and file, if they were open.
   if ( server_ != NULL )
   {
      server_->Delete();
      wxLogMessage(wxT(" packetlog: log server stopped."));
   }
   if ( file_ != NULL )
   {
      delete file_;
      wxLogMessage(wxT(" packetlog: log file closed."));
   }
}

bool Packetlog::operator!() const
{
   return !initialized_;
}

bool Packetlog::reopen()
{
   if ( fileNeeded_ )
   {
      delete file_;
      file_ = new Logfile(cfg_.packetFileName.ToAscii());
      if ( !(*file_) )
      {
         wxLogMessage(wxT(" packetlog: failed to reopen log file."));
         return false;
      }
      else
      {
         wxLogMessage(wxT(" packetlog: log file reopened."));
      }
   }
   
   return true;
}

void Packetlog::write(Packet* packet)
{
   // Here will the log line be created.
   ostringstream line;
   
   // Construt a nice timestamp text.
   wxDateTime timestamp = wxDateTime::Now();
   wxString timetext = timestamp.FormatISODate();
   timetext += wxT(" ");
   timetext += timestamp.FormatISOTime();
   
   // Construct the line.
   line << timetext.ToAscii() << " i" << packet->input << " o" << packet->outputs << " " << packet->flag << " " << std::setw(3)
     << packet->aprsPacket->getTnc2Packet().size() << "B " << packet->aprsPacket->getTnc2Packet();
   
   string msg;
   if ( packet->aprsPacket->error(msg) )
   {
      line << " (" << msg << ")";
   }
   
   // Write the line.
   if ( file_ != NULL )
   {
      file_->write( line.str() + "\n" );
   }
   if ( server_ != NULL )
   {
      server_->write( line.str() + "\r\n" );
   }
   
   // Packet has been logged, so let's erase it.
   delete packet->aprsPacket;
   delete packet->thirdParty;
   delete packet;
}
