/* $Id: packetlog.h 322 2009-10-19 16:43:34Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 *
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef PACKETLOG_HH
#define PACKETLOG_HH

// own stuff.
#include "logfile.h"
#include "logserver.h"
#include "cfg.h"
#include "packet.h"
#include "aprspacket.h"

// std stuff.
#include <string>
#include <list>

// wx stuff.
#include <wx/string.h>
#include <wx/socket.h>


/// Packet logging manager. Can be configured to log into files and/or tcp socket.
class Packetlog
{
 public:
      
   /// Constructor.
   /**
    * Opens files and sockets. Remember to use operator! to check how it went.
    * \param cfg Tells us where to log.
    * \param ownCall Callsign of this IGate.
    * \param ownLat Latitude from config file.
    * \param ownLon Longitude from config file.
    */
   Packetlog(Cfg::LogCfg const& cfg, wxString const& ownCall,
             AprsPacket::Location const& ownLoc);
   
   /// Destructor.
   /**
    * Closes files and sockets.
    */
   ~Packetlog();
   
   /// Status query.
   /**
    * \return true Everything is working as configured.
    * \return false There's a fatal error.
    */
   bool operator!() const;
   
   /// Refreshing operator.
   /**
    * Reopens the log file, if open.
    * \return false if file was open but can't be reopened.
    * \return true in any other case.
    */
   bool reopen();
   
   /// Writes given packet into the logs.
   /**
    * Memory occupied by packet is freed after write has completed.
    */
   void write(Packet* packet);
   
 private:
   
   /// Flag set by constructor.
   bool initialized_;
   
   /// Our config.
   Cfg::LogCfg const& cfg_;
   
   /// Wether to open log file or not.
   bool fileNeeded_;
   
   /// Log file.
   /**
    * File nees to be handled dynamically because we need to be able to reopen it.
    */
   Logfile* file_;
   
   /// Log server.
   /**
    * Server needs to be handled dynamically since it deletes itself upon termination.
    */
   LogServer* server_;
   
};

#endif // PACKETLOG_HH
