/* $Id: serial.h 321 2009-10-19 14:25:40Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef SERIAL_HH
#define SERIAL_HH

// Own stuff.
#include "cfg.h"
#include "connection.h"
#include "serialport.h"

// std stuff.
#include <string>
#include <vector>

// wx stuff.
#include <wx/string.h>
#include <wx/thread.h>
#include <wx/file.h>


/// Common functionality of serial port based connections.
/**
 * Handles serial port device. Reports errors on serial port operations
 * via wxLogMessage.
 */
class Serial : public Connection
{
 public:
   
   /// Constructor tries to open the device and calls connected().
   Serial(wxString const& deviceName, Cfg::ConnectionState const& state,
                    Cfg::SerialCfg const& portcfg, wxString const& lockdir);

   /// Destructor calls removeLockFile() and closes the device.
   virtual ~Serial();

   // Implementation of the "main()" of each thread is forwarded to child classes.
   virtual void* Entry() = 0;
   
 protected:
   
   /// Read bytes from serial port.
   /**
    * \param data Bytes read from the device or an empty string in case of timeout.
    * \return True in case of bytes received or timeout, false if error.
    */
   bool readDevice(std::string& data);
   
   /// Write given bytes to serial port.
   /**
    * \return True if data was written successfully, false otherwise.
    */
   bool writeToDevice(std::string const& data);
   
   /// Tells our children how the serial port is doing.
   bool portIsConnected() const;
   
 private:
   
   /// Create a lock file for our port device.
   /**
    * \return False, if lock file already exists or cannot be created. True on success.
    */
   bool createLockFile();  
   
   /// Removes our lock file.
   void removeLockFile();
   
   /// Name of the lock file.
   wxString lockFileName_;
   
   /// Lock file handle.
   wxFile* lockFile_;
   
   
   /// File name of the serial port device.
   wxString deviceName_;
   
   /// The serial port device itself.
   Serialport* device_;

};

#endif // SERIAL_HH
