/* $Id: serialport.h 319 2009-10-19 14:06:42Z oh2gve $
 * 
 * Copyright 2009 Tapio Aaltonen
 * 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SERIALPORT_HH
#define SERIALPORT_HH


// std stuff.
#include <string>



/// Represents one serial port device.
class Serialport
{
 public:
      
   /// Constructor.
   Serialport(std::string const& devicefile, unsigned int const& baudrate,
              unsigned int const& charsize, char const& parity, unsigned int const& stopbits,
              bool const& rtscts);
   
   /// Destructor.
   ~Serialport();
   
   /// Writes given data to the port.
   /**
    * \return True if all data was written, false in other cases.
    */
   bool transmit(std::string const& data);
   
   /// Reads bytes from input buffer or timeouts after 100 ms if buffer is empty.
   /**
    * \return True in case of received data or timeout, false if error happened.
    */
   bool receive(std::string& data);
   
   /// Check port status.
   bool operator!() const;
   
   /// Returns error message.
   std::string error() const;
   
   /// Writes to 
   
 private:
   
   /// File descriptor of the device.
   int fd_;
   
   /// In case of error the message is stored here.
   std::string error_;
};

#endif // SERIALPORT_HH
